
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	CAMERA.CPP                                                          //
//                                                                          //
//	Implementation of CAMERA class.                                     //
//                    							    //
//////////////////////////////////////////////////////////////////////////////

#include "CAMERA\CAMERA.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Constructor creates a camera at 0,0,0 with orientation 0,0,0             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

CAMERA :: CAMERA ( void )

/*	PRE	:	TRUE
	POST	:	Camera has default position and orientation
*/

{
	/*	Set the default
		orientation.	*/

	Pitch = 0.0;
	Roll = 0.0;
	Yaw = 0.0;

	/*	Set the default
		position.	*/

	Location.v [0] = 0.0;
	Location.v [1] = 0.0;
	Location.v [2] = 0.0;	
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to increase the roll of the camera.                             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: RollBy ( REAL Amount )

/*	PRE	:
	POST	:
*/

{
	Roll += Amount;
	if ( Roll > 360.0 ) {
		Roll -= 360.0;
	} else {
		if ( Roll < 0.0 ) {
			Roll += 360.0;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to increase the pitch of the camera.                            //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: PitchBy ( REAL Amount )

/*	PRE	:
	POST	:
*/

{
	Pitch += Amount;
	if ( Pitch > 360.0 ) {
		Pitch -= 360.0;
	} else {
		if ( Pitch < 0.0 ) {
			Pitch += 360.0;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to increase the yaw of the camera.                              //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: YawBy ( REAL Amount )

/*	PRE	:
	POST	:
*/

{
	Yaw += Amount;
	if ( Yaw > 360.0 ) {
		Yaw -= 360.0;
	} else {
		if ( Yaw < 0.0 ) {
			Yaw += 360.0;
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Readies the camera for transforming points.                              //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: Ready ( void )

/*	PRE	:	TRUE
	POST	:	Sets up the transformation matrix for this camera.
*/

{
	MATRIX	mPitch, mRoll, mYaw;
	REAL	p, r, y;

	/*	Get the p, r and y
		in radians.	*/

	p = RADIAN ( Pitch );
	r = RADIAN ( Roll );
	y = RADIAN ( Yaw );

	/*	Load the pitch	
		matrix.	*/

	mPitch.Pitch ( p );

	/*	Load the roll
		matrix.	*/

	mRoll.Roll ( r );

	/*	Load the yaw
		matrix.	*/

	mYaw.Yaw ( y );

	/*	Conacentate
		the roll and yaw	*/

	MATRIX mTransient1 ( mYaw, mRoll );

	/*	And concatenate
		the pitch.	*/

	MATRIX mTransient2 ( mPitch, mTransient1 );

	/*	Store the result
		into the transformation.	*/

	Transformation = mTransient2;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to return the angle between a camera and a plane.               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

REAL CAMERA :: GetSurfaceAngle ( POINT *p0, POINT *p1, POINT *p2 )

/*	PRE	:	Points are 3 consecutive points on a plane
	POST	:	Returns the angle of the plane w.r.t. the camera.
*/

{
	/*	Make a vector from p1 
		to p0.	*/

	VECTOR a ( p1, p0 );

	/*	Make a vector from p1
		to p2.	*/

	VECTOR b ( p1, p2 );

	/*	Make a vector from p1
		to camera.	*/

	VECTOR o ( p1, &Location );

	/*	Find the surface
		normal ( in `a' ).	*/

	a.Cross ( b );

	/*	Make the normal
		unit length.	*/

	a.Unit ();

	/*	Make the observer
		vector unit length.	*/

	o.Unit ();

	/*	Take the dot product
		of this and the normal.	*/

	return ( a.Dot ( o ) );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to return -1, 0 or 1 depending on surface orientation.          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int CAMERA :: GetOrientation ( POINT *p0, POINT *p1, POINT *p2 )

/*	PRE	:	Points are 3 consecutive points on a plane
	POST	:	Returns the -1, 0 or 1 depending on angle of plane
*/

{
	/*	Make a vector from p1 
		to p0.	*/

	VECTOR a ( p1, p0 );

	/*	Make a vector from p1
		to p2.	*/

	VECTOR b ( p1, p2 );

	/*	Make a vector from p1
		to camera.	*/

	VECTOR o ( p1, &Location );

	/*	Find the surface
		normal ( in `a' ).	*/

	a.Cross ( b );

	/*	Take the dot product
		of this and the normal.	*/

	return ( ( int ) SGN ( a.Dot ( o ) ) );
}

PIXEL CAMERA :: GetLightIntensity ( POINT *p1, POINT *p2, POINT *p3, PIXEL Colour )

/*	PRE	:
	POST	:
*/

{
	REAL Angle;

	Angle = GetSurfaceAngle ( p1, p2, p3 );
	return ( Colour + ( int ) ( 16.0 * ( ABS ( Angle ) ) ) );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Sets the angle of orientation of the camera ( pitch, roll, yaw ).        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: SetAngles ( REAL p, REAL r, REAL y )

/*	PRE	:	TRUE
	POST	:	Orientation of camera is ( p, r, y )
*/

{
	Pitch = p;
	Roll = r;
	Yaw = y;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Moves the camera to the given location in space.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: SetLocation ( REAL x, REAL y, REAL z )

/*	PRE	:	TRUE
	POST	:	Position of camera = p
*/

{
	/*	We will translate the world by 
		( the location of the camera. ).	*/

	Location.v [0] = x;
	Location.v [1] = y;
	Location.v [2] = z;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return a reference to the location of the camera in space.               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

POINT& CAMERA :: GetLocation ( void )

/*	PRE	:	TRUE
	POST	:	RETURNS Position of camera.
*/

{
	return ( Location );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Moves the camera to the given location in space.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

MATRIX& CAMERA :: GetMatrix ( void )

/*	PRE	:	TRUE
	POST	:	The transformation matrix is returned.
*/

{
	return ( Transformation );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return the pitch, roll and yaw values for the camera.                    //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void CAMERA :: GetPitchRollYaw ( REAL *p, REAL *r, REAL *y )

/*	PRE	:	TRUE
	POST	:	Return pitch, roll and yaw
*/

{
	*p = Pitch;
	*r = Roll;
	*y = Yaw;
}

