
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	VIEW.CPP                                                            //
//                                                                          //
//	Implementation of the VIEWPROJECTION class.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#include "VIEW\VIEW.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to clip a polygon to the view frustum and render it.           //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VIEWPROJECTION :: ClipShadeAndRender ( GRAPHICSMODE& Buffer, BUCKETPOLYGON *Polygon, POINT *Vertices )

/*	PRE	:	TRUE
	POST	:	Clips and renders the polygon.
*/

{
	static	POINT	 In [MAXPOLYGONVERTEX], Out [MAXPOLYGONVERTEX];
	static	INTPOINT Integer [MAXPOLYGONVERTEX], OutTex [MAXPOLYGONVERTEX];
	int		 i, nVerts;
	PIXEL		 Colour;
	POLYGON		 *p = Polygon->p;

	/*	Copy the vertices into 
		the temporary structure.	*/

	for ( i = 0; i < p->nVerts; i++ ) {
		In [i] = Vertices [ p->Vertices [i] ];
	}

        /*	Clip to 
		frustum.	*/

	nVerts = p->nVerts;
	if ( ClipToViewFrustum ( &nVerts, In, Out, Polygon->TexVerts, OutTex ) == TRUE ) {

		/*	Get the colour of the surface
			w.r.t. the light source.	*/

		Colour = cLight->GetLightIntensity ( &Out [0], &Out [1], &Out [2], Polygon->Colour );

		/*	Normalise to the
			window.	*/

		Project ( nVerts, Out, Integer );

		/*	And finally, render 
			to the buffer.	*/

		if ( ( p->Flag & TEXTURED ) == TEXTURED ) {
			if ( ( p->Flag & TRANSPARENT ) == TRANSPARENT ) {
				Buffer.SetTexture ( Polygon->Texture, TRUE );
			} else {	
				Buffer.SetTexture ( Polygon->Texture, FALSE );
			}
			Buffer.TextureMapPolygon ( nVerts, Integer, OutTex, Polygon->Culled, Colour );
		} else {
			if ( ( p->Flag & FLATSHADED ) == FLATSHADED ) {
				Buffer.FlatShadePolygon ( nVerts, Integer, Colour );
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to clip a polygon to the viewing frustum.                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

BOOL VIEWPROJECTION :: ClipToViewFrustum ( int *nVerts, POINT *In, POINT *Out, INTPOINT *InTex, INTPOINT *OutTex )

/*	PRE	:
	POST	:
*/

{
	static POINT	Temp	[2][MAXPOLYGONVERTEX];
	static INTPOINT Tex	[2][MAXPOLYGONVERTEX];
	POINT			*Vertices = In;
	INTPOINT		*tVertices = InTex;
	int			CurPoly = 0, i;
	BOOL			Result;

	/*	Clip the polygon to
		each plane.	*/

	if ( ( InTex == NULL ) || ( OutTex == NULL ) ) {
		for ( i = 0; i < ( NUMFRUSTUMPLANES - 1 ); i++ ) {

			/*	Clip the polygon
				to this plane.       	*/

			Result = Frustum [i].Clip ( nVerts, Vertices, Temp [CurPoly], NULL, NULL );
			if ( Result == FALSE ) {
				return ( FALSE );
			}

			/*	Flip the in\out
				set.	*/

			Vertices = Temp [CurPoly];
			CurPoly ^= 1;
		}

		/*	Return result of clipping
			to final clipping plane.	*/

		return ( Frustum [i].Clip ( nVerts, Vertices, Out, NULL, NULL ) );
	} else {
		for ( i = 0; i < ( NUMFRUSTUMPLANES - 1 ); i++ ) {

			/*	Clip the polygon
				to this plane.       	*/

			Result = Frustum [i].Clip ( nVerts, Vertices, Temp [CurPoly], tVertices, Tex [CurPoly] );
			if ( Result == FALSE ) {
				return ( FALSE );
			}

			/*	Flip the in\out
				set.	*/

			Vertices = Temp [CurPoly];
			tVertices = Tex [CurPoly];
			CurPoly ^= 1;
		}

		/*	Return result of clipping
			to final clipping plane.	*/

		return ( Frustum [i].Clip ( nVerts, Vertices, Out, tVertices, OutTex ) );
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Sets up the viewspace frustum ready for clipping.                        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VIEWPROJECTION :: SetupFrustum ( WINDOW& Win )

/*	PRE	:
	POST	:
*/

{
	int wLeft, wTop, wRight, wBottom, wWidth, wLength;

	/*	Set the current
		window for the view.	*/

	Window = &Win;

	/*	Customarily set the
		field of view to 2.0.	*/

	FieldOfView = 2.0;

	/*	and the top left
		position.	*/

	Window->GetArea ( &wLeft, &wTop, &wRight, &wBottom );

	/*	Calc the size
		of the window.	*/

	wWidth = wRight - wLeft;
	wLength = wBottom - wTop;

	/*	Find out the x and
		y scaling factors.	*/

	xScale = ( REAL ) wWidth / FieldOfView;
	yScale = ( REAL ) wLength / FieldOfView;
	maxScale = MAX ( xScale, yScale );

	/*	And the center
		of projection.	*/

	xCenter = ( REAL ) wWidth / 2.0 - 0.5;
	yCenter = ( REAL ) wLength / 2.0 + 0.5;

	REAL	Angle, Sin, Cos;
	VECTOR	Normal;
	POINT	Point ( 0.0, 0.0, 0.0 );

	Angle = atan ( 2.0 / FieldOfView * maxScale / xScale );
	Sin = sin ( Angle );
	Cos = cos ( Angle );

	Normal.Set ( -Sin, 0.0, -Cos );
	Frustum [0].Set ( Normal, Point );
	Normal.Set ( Sin, 0.0, -Cos );
	Frustum [1].Set ( Normal, Point );

	Angle = atan ( 2.0 / FieldOfView * maxScale / yScale );
	Sin = sin ( Angle );
	Cos = cos ( Angle );

	Normal.Set ( 0.0, Sin, -Cos );
	Frustum [2].Set ( Normal, Point );
	Normal.Set ( 0.0, -Sin, -Cos );
	Frustum [3].Set ( Normal, Point );
}

